/*
Metal API

# Introduction Equinix Metal provides a RESTful HTTP API which can be reached at <https://api.equinix.com/metal/v1>. This document describes the API and how to use it.  The API allows you to programmatically interact with all of your Equinix Metal resources, including devices, networks, addresses, organizations, projects, and your user account. Every feature of the Equinix Metal web interface is accessible through the API.  The API docs are generated from the Equinix Metal OpenAPI specification and are officially hosted at <https://metal.equinix.com/developers/api>.  # Common Parameters  The Equinix Metal API uses a few methods to minimize network traffic and improve throughput. These parameters are not used in all API calls, but are used often enough to warrant their own section. Look for these parameters in the documentation for the API calls that support them.  ## Pagination  Pagination is used to limit the number of results returned in a single request. The API will return a maximum of 100 results per page. To retrieve additional results, you can use the `page` and `per_page` query parameters.  The `page` parameter is used to specify the page number. The first page is `1`. The `per_page` parameter is used to specify the number of results per page. The maximum number of results differs by resource type.  ## Sorting  Where offered, the API allows you to sort results by a specific field. To sort results use the `sort_by` query parameter with the root level field name as the value. The `sort_direction` parameter is used to specify the sort direction, either either `asc` (ascending) or `desc` (descending).  ## Filtering  Filtering is used to limit the results returned in a single request. The API supports filtering by certain fields in the response. To filter results, you can use the field as a query parameter.  For example, to filter the IP list to only return public IPv4 addresses, you can filter by the `type` field, as in the following request:  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/projects/id/ips?type=public_ipv4 ```  Only IP addresses with the `type` field set to `public_ipv4` will be returned.  ## Searching  Searching is used to find matching resources using multiple field comparissons. The API supports searching in resources that define this behavior. Currently the search parameter is only available on devices, ssh_keys, api_keys and memberships endpoints.  To search resources you can use the `search` query parameter.  ## Include and Exclude  For resources that contain references to other resources, sucha as a Device that refers to the Project it resides in, the Equinix Metal API will returns `href` values (API links) to the associated resource.  ```json {   ...   \"project\": {     \"href\": \"/metal/v1/projects/f3f131c8-f302-49ef-8c44-9405022dc6dd\"   } } ```  If you're going need the project details, you can avoid a second API request.  Specify the contained `href` resources and collections that you'd like to have included in the response using the `include` query parameter.  For example:  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=projects ```  The `include` parameter is generally accepted in `GET`, `POST`, `PUT`, and `PATCH` requests where `href` resources are presented.  To have multiple resources include, use a comma-separated list (e.g. `?include=emails,projects,memberships`).  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=emails,projects,memberships ```  You may also include nested associations up to three levels deep using dot notation (`?include=memberships.projects`):  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=memberships.projects ```  To exclude resources, and optimize response delivery, use the `exclude` query parameter. The `exclude` parameter is generally accepted in `GET`, `POST`, `PUT`, and `PATCH` requests for fields with nested object responses. When excluded, these fields will be replaced with an object that contains only an `href` field.

API version: 1.0.0
Contact: support@equinixmetal.com
*/

// Code generated by OpenAPI Generator (https://openapi-generator.tech); DO NOT EDIT.

package metalv1

import (
	"encoding/json"
	"time"
)

// checks if the Device type satisfies the MappedNullable interface at compile time
var _ MappedNullable = &Device{}

// Device struct for Device
type Device struct {
	AlwaysPxe    *bool                  `json:"always_pxe,omitempty"`
	BillingCycle *string                `json:"billing_cycle,omitempty"`
	BondingMode  *int32                 `json:"bonding_mode,omitempty"`
	CreatedAt    *time.Time             `json:"created_at,omitempty"`
	CreatedBy    *DeviceCreatedBy       `json:"created_by,omitempty"`
	Customdata   map[string]interface{} `json:"customdata,omitempty"`
	Description  *string                `json:"description,omitempty"`
	Facility     *Facility              `json:"facility,omitempty"`
	// The UUID of the firmware set to associate with the device.
	FirmwareSetId       *string              `json:"firmware_set_id,omitempty"`
	HardwareReservation *HardwareReservation `json:"hardware_reservation,omitempty"`
	Hostname            *string              `json:"hostname,omitempty"`
	Href                *string              `json:"href,omitempty"`
	Id                  *string              `json:"id,omitempty"`
	ImageUrl            *string              `json:"image_url,omitempty"`
	IpAddresses         []IPAssignment       `json:"ip_addresses,omitempty"`
	IpxeScriptUrl       *string              `json:"ipxe_script_url,omitempty"`
	Iqn                 *string              `json:"iqn,omitempty"`
	// Prevents accidental deletion of this resource when set to true.
	Locked *bool        `json:"locked,omitempty"`
	Metro  *DeviceMetro `json:"metro,omitempty"`
	// Whether network mode changes such as converting to/from Layer2 or Layer3 networking, bonding or disbonding network interfaces are permitted for the device.
	NetworkFrozen *bool `json:"network_frozen,omitempty"`
	// By default, servers at Equinix Metal are configured in a “bonded” mode using LACP (Link Aggregation Control Protocol). Each 2-NIC server is configured with a single bond (namely bond0) with both interfaces eth0 and eth1 as members of the bond in a default Layer 3 mode. Some device plans may have a different number of ports and bonds available.
	NetworkPorts    []Port           `json:"network_ports,omitempty"`
	OperatingSystem *OperatingSystem `json:"operating_system,omitempty"`
	// Actions supported by the device instance.
	Actions            []DeviceActionsInner `json:"actions,omitempty"`
	Plan               *Plan                `json:"plan,omitempty"`
	Project            *Project             `json:"project,omitempty"`
	ProjectLite        *DeviceProjectLite   `json:"project_lite,omitempty"`
	ProvisioningEvents []Event              `json:"provisioning_events,omitempty"`
	// Only visible while device provisioning
	ProvisioningPercentage *float32 `json:"provisioning_percentage,omitempty"`
	// Root password is automatically generated when server is provisioned and it is removed after 24 hours
	RootPassword *string `json:"root_password,omitempty"`
	ShortId      *string `json:"short_id,omitempty"`
	// Whether or not the device is a spot instance.
	SpotInstance *bool `json:"spot_instance,omitempty"`
	// The maximum price per hour you are willing to pay to keep this spot instance.  If you are outbid, the termination will be set allowing two minutes before shutdown.
	SpotPriceMax *float32     `json:"spot_price_max,omitempty"`
	SshKeys      []Href       `json:"ssh_keys,omitempty"`
	State        *DeviceState `json:"state,omitempty"`
	Storage      *Storage     `json:"storage,omitempty"`
	// Switch short id. This can be used to determine if two devices are connected to the same switch, for example.
	SwitchUuid *string  `json:"switch_uuid,omitempty"`
	Tags       []string `json:"tags,omitempty"`
	// When the device will be terminated. If you don't supply timezone info, the timestamp is assumed to be in UTC.  This is commonly set in advance for ephemeral spot market instances but this field may also be set with on-demand and reservation instances to automatically delete the resource at a given time. The termination time can also be used to release a hardware reservation instance at a given time, keeping the reservation open for other uses.  On a spot market device, the termination time will be set automatically when outbid.
	TerminationTime *time.Time `json:"termination_time,omitempty"`
	UpdatedAt       *time.Time `json:"updated_at,omitempty"`
	User            *string    `json:"user,omitempty"`
	Userdata        *string    `json:"userdata,omitempty"`
	Volumes         []Href     `json:"volumes,omitempty"`
	// Hostname used to connect to the instance via the SOS (Serial over SSH) out-of-band console.
	Sos                  *string `json:"sos,omitempty"`
	AdditionalProperties map[string]interface{}
}

type _Device Device

// NewDevice instantiates a new Device object
// This constructor will assign default values to properties that have it defined,
// and makes sure properties required by API are set, but the set of arguments
// will change when the set of required properties is changed
func NewDevice() *Device {
	this := Device{}
	return &this
}

// NewDeviceWithDefaults instantiates a new Device object
// This constructor will only assign default values to properties that have it defined,
// but it doesn't guarantee that properties required by API are set
func NewDeviceWithDefaults() *Device {
	this := Device{}
	return &this
}

// GetAlwaysPxe returns the AlwaysPxe field value if set, zero value otherwise.
func (o *Device) GetAlwaysPxe() bool {
	if o == nil || IsNil(o.AlwaysPxe) {
		var ret bool
		return ret
	}
	return *o.AlwaysPxe
}

// GetAlwaysPxeOk returns a tuple with the AlwaysPxe field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetAlwaysPxeOk() (*bool, bool) {
	if o == nil || IsNil(o.AlwaysPxe) {
		return nil, false
	}
	return o.AlwaysPxe, true
}

// HasAlwaysPxe returns a boolean if a field has been set.
func (o *Device) HasAlwaysPxe() bool {
	if o != nil && !IsNil(o.AlwaysPxe) {
		return true
	}

	return false
}

// SetAlwaysPxe gets a reference to the given bool and assigns it to the AlwaysPxe field.
func (o *Device) SetAlwaysPxe(v bool) {
	o.AlwaysPxe = &v
}

// GetBillingCycle returns the BillingCycle field value if set, zero value otherwise.
func (o *Device) GetBillingCycle() string {
	if o == nil || IsNil(o.BillingCycle) {
		var ret string
		return ret
	}
	return *o.BillingCycle
}

// GetBillingCycleOk returns a tuple with the BillingCycle field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetBillingCycleOk() (*string, bool) {
	if o == nil || IsNil(o.BillingCycle) {
		return nil, false
	}
	return o.BillingCycle, true
}

// HasBillingCycle returns a boolean if a field has been set.
func (o *Device) HasBillingCycle() bool {
	if o != nil && !IsNil(o.BillingCycle) {
		return true
	}

	return false
}

// SetBillingCycle gets a reference to the given string and assigns it to the BillingCycle field.
func (o *Device) SetBillingCycle(v string) {
	o.BillingCycle = &v
}

// GetBondingMode returns the BondingMode field value if set, zero value otherwise.
func (o *Device) GetBondingMode() int32 {
	if o == nil || IsNil(o.BondingMode) {
		var ret int32
		return ret
	}
	return *o.BondingMode
}

// GetBondingModeOk returns a tuple with the BondingMode field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetBondingModeOk() (*int32, bool) {
	if o == nil || IsNil(o.BondingMode) {
		return nil, false
	}
	return o.BondingMode, true
}

// HasBondingMode returns a boolean if a field has been set.
func (o *Device) HasBondingMode() bool {
	if o != nil && !IsNil(o.BondingMode) {
		return true
	}

	return false
}

// SetBondingMode gets a reference to the given int32 and assigns it to the BondingMode field.
func (o *Device) SetBondingMode(v int32) {
	o.BondingMode = &v
}

// GetCreatedAt returns the CreatedAt field value if set, zero value otherwise.
func (o *Device) GetCreatedAt() time.Time {
	if o == nil || IsNil(o.CreatedAt) {
		var ret time.Time
		return ret
	}
	return *o.CreatedAt
}

// GetCreatedAtOk returns a tuple with the CreatedAt field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetCreatedAtOk() (*time.Time, bool) {
	if o == nil || IsNil(o.CreatedAt) {
		return nil, false
	}
	return o.CreatedAt, true
}

// HasCreatedAt returns a boolean if a field has been set.
func (o *Device) HasCreatedAt() bool {
	if o != nil && !IsNil(o.CreatedAt) {
		return true
	}

	return false
}

// SetCreatedAt gets a reference to the given time.Time and assigns it to the CreatedAt field.
func (o *Device) SetCreatedAt(v time.Time) {
	o.CreatedAt = &v
}

// GetCreatedBy returns the CreatedBy field value if set, zero value otherwise.
func (o *Device) GetCreatedBy() DeviceCreatedBy {
	if o == nil || IsNil(o.CreatedBy) {
		var ret DeviceCreatedBy
		return ret
	}
	return *o.CreatedBy
}

// GetCreatedByOk returns a tuple with the CreatedBy field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetCreatedByOk() (*DeviceCreatedBy, bool) {
	if o == nil || IsNil(o.CreatedBy) {
		return nil, false
	}
	return o.CreatedBy, true
}

// HasCreatedBy returns a boolean if a field has been set.
func (o *Device) HasCreatedBy() bool {
	if o != nil && !IsNil(o.CreatedBy) {
		return true
	}

	return false
}

// SetCreatedBy gets a reference to the given DeviceCreatedBy and assigns it to the CreatedBy field.
func (o *Device) SetCreatedBy(v DeviceCreatedBy) {
	o.CreatedBy = &v
}

// GetCustomdata returns the Customdata field value if set, zero value otherwise.
func (o *Device) GetCustomdata() map[string]interface{} {
	if o == nil || IsNil(o.Customdata) {
		var ret map[string]interface{}
		return ret
	}
	return o.Customdata
}

// GetCustomdataOk returns a tuple with the Customdata field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetCustomdataOk() (map[string]interface{}, bool) {
	if o == nil || IsNil(o.Customdata) {
		return map[string]interface{}{}, false
	}
	return o.Customdata, true
}

// HasCustomdata returns a boolean if a field has been set.
func (o *Device) HasCustomdata() bool {
	if o != nil && !IsNil(o.Customdata) {
		return true
	}

	return false
}

// SetCustomdata gets a reference to the given map[string]interface{} and assigns it to the Customdata field.
func (o *Device) SetCustomdata(v map[string]interface{}) {
	o.Customdata = v
}

// GetDescription returns the Description field value if set, zero value otherwise.
func (o *Device) GetDescription() string {
	if o == nil || IsNil(o.Description) {
		var ret string
		return ret
	}
	return *o.Description
}

// GetDescriptionOk returns a tuple with the Description field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetDescriptionOk() (*string, bool) {
	if o == nil || IsNil(o.Description) {
		return nil, false
	}
	return o.Description, true
}

// HasDescription returns a boolean if a field has been set.
func (o *Device) HasDescription() bool {
	if o != nil && !IsNil(o.Description) {
		return true
	}

	return false
}

// SetDescription gets a reference to the given string and assigns it to the Description field.
func (o *Device) SetDescription(v string) {
	o.Description = &v
}

// GetFacility returns the Facility field value if set, zero value otherwise.
func (o *Device) GetFacility() Facility {
	if o == nil || IsNil(o.Facility) {
		var ret Facility
		return ret
	}
	return *o.Facility
}

// GetFacilityOk returns a tuple with the Facility field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetFacilityOk() (*Facility, bool) {
	if o == nil || IsNil(o.Facility) {
		return nil, false
	}
	return o.Facility, true
}

// HasFacility returns a boolean if a field has been set.
func (o *Device) HasFacility() bool {
	if o != nil && !IsNil(o.Facility) {
		return true
	}

	return false
}

// SetFacility gets a reference to the given Facility and assigns it to the Facility field.
func (o *Device) SetFacility(v Facility) {
	o.Facility = &v
}

// GetFirmwareSetId returns the FirmwareSetId field value if set, zero value otherwise.
func (o *Device) GetFirmwareSetId() string {
	if o == nil || IsNil(o.FirmwareSetId) {
		var ret string
		return ret
	}
	return *o.FirmwareSetId
}

// GetFirmwareSetIdOk returns a tuple with the FirmwareSetId field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetFirmwareSetIdOk() (*string, bool) {
	if o == nil || IsNil(o.FirmwareSetId) {
		return nil, false
	}
	return o.FirmwareSetId, true
}

// HasFirmwareSetId returns a boolean if a field has been set.
func (o *Device) HasFirmwareSetId() bool {
	if o != nil && !IsNil(o.FirmwareSetId) {
		return true
	}

	return false
}

// SetFirmwareSetId gets a reference to the given string and assigns it to the FirmwareSetId field.
func (o *Device) SetFirmwareSetId(v string) {
	o.FirmwareSetId = &v
}

// GetHardwareReservation returns the HardwareReservation field value if set, zero value otherwise.
func (o *Device) GetHardwareReservation() HardwareReservation {
	if o == nil || IsNil(o.HardwareReservation) {
		var ret HardwareReservation
		return ret
	}
	return *o.HardwareReservation
}

// GetHardwareReservationOk returns a tuple with the HardwareReservation field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetHardwareReservationOk() (*HardwareReservation, bool) {
	if o == nil || IsNil(o.HardwareReservation) {
		return nil, false
	}
	return o.HardwareReservation, true
}

// HasHardwareReservation returns a boolean if a field has been set.
func (o *Device) HasHardwareReservation() bool {
	if o != nil && !IsNil(o.HardwareReservation) {
		return true
	}

	return false
}

// SetHardwareReservation gets a reference to the given HardwareReservation and assigns it to the HardwareReservation field.
func (o *Device) SetHardwareReservation(v HardwareReservation) {
	o.HardwareReservation = &v
}

// GetHostname returns the Hostname field value if set, zero value otherwise.
func (o *Device) GetHostname() string {
	if o == nil || IsNil(o.Hostname) {
		var ret string
		return ret
	}
	return *o.Hostname
}

// GetHostnameOk returns a tuple with the Hostname field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetHostnameOk() (*string, bool) {
	if o == nil || IsNil(o.Hostname) {
		return nil, false
	}
	return o.Hostname, true
}

// HasHostname returns a boolean if a field has been set.
func (o *Device) HasHostname() bool {
	if o != nil && !IsNil(o.Hostname) {
		return true
	}

	return false
}

// SetHostname gets a reference to the given string and assigns it to the Hostname field.
func (o *Device) SetHostname(v string) {
	o.Hostname = &v
}

// GetHref returns the Href field value if set, zero value otherwise.
func (o *Device) GetHref() string {
	if o == nil || IsNil(o.Href) {
		var ret string
		return ret
	}
	return *o.Href
}

// GetHrefOk returns a tuple with the Href field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetHrefOk() (*string, bool) {
	if o == nil || IsNil(o.Href) {
		return nil, false
	}
	return o.Href, true
}

// HasHref returns a boolean if a field has been set.
func (o *Device) HasHref() bool {
	if o != nil && !IsNil(o.Href) {
		return true
	}

	return false
}

// SetHref gets a reference to the given string and assigns it to the Href field.
func (o *Device) SetHref(v string) {
	o.Href = &v
}

// GetId returns the Id field value if set, zero value otherwise.
func (o *Device) GetId() string {
	if o == nil || IsNil(o.Id) {
		var ret string
		return ret
	}
	return *o.Id
}

// GetIdOk returns a tuple with the Id field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetIdOk() (*string, bool) {
	if o == nil || IsNil(o.Id) {
		return nil, false
	}
	return o.Id, true
}

// HasId returns a boolean if a field has been set.
func (o *Device) HasId() bool {
	if o != nil && !IsNil(o.Id) {
		return true
	}

	return false
}

// SetId gets a reference to the given string and assigns it to the Id field.
func (o *Device) SetId(v string) {
	o.Id = &v
}

// GetImageUrl returns the ImageUrl field value if set, zero value otherwise.
func (o *Device) GetImageUrl() string {
	if o == nil || IsNil(o.ImageUrl) {
		var ret string
		return ret
	}
	return *o.ImageUrl
}

// GetImageUrlOk returns a tuple with the ImageUrl field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetImageUrlOk() (*string, bool) {
	if o == nil || IsNil(o.ImageUrl) {
		return nil, false
	}
	return o.ImageUrl, true
}

// HasImageUrl returns a boolean if a field has been set.
func (o *Device) HasImageUrl() bool {
	if o != nil && !IsNil(o.ImageUrl) {
		return true
	}

	return false
}

// SetImageUrl gets a reference to the given string and assigns it to the ImageUrl field.
func (o *Device) SetImageUrl(v string) {
	o.ImageUrl = &v
}

// GetIpAddresses returns the IpAddresses field value if set, zero value otherwise.
func (o *Device) GetIpAddresses() []IPAssignment {
	if o == nil || IsNil(o.IpAddresses) {
		var ret []IPAssignment
		return ret
	}
	return o.IpAddresses
}

// GetIpAddressesOk returns a tuple with the IpAddresses field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetIpAddressesOk() ([]IPAssignment, bool) {
	if o == nil || IsNil(o.IpAddresses) {
		return nil, false
	}
	return o.IpAddresses, true
}

// HasIpAddresses returns a boolean if a field has been set.
func (o *Device) HasIpAddresses() bool {
	if o != nil && !IsNil(o.IpAddresses) {
		return true
	}

	return false
}

// SetIpAddresses gets a reference to the given []IPAssignment and assigns it to the IpAddresses field.
func (o *Device) SetIpAddresses(v []IPAssignment) {
	o.IpAddresses = v
}

// GetIpxeScriptUrl returns the IpxeScriptUrl field value if set, zero value otherwise.
func (o *Device) GetIpxeScriptUrl() string {
	if o == nil || IsNil(o.IpxeScriptUrl) {
		var ret string
		return ret
	}
	return *o.IpxeScriptUrl
}

// GetIpxeScriptUrlOk returns a tuple with the IpxeScriptUrl field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetIpxeScriptUrlOk() (*string, bool) {
	if o == nil || IsNil(o.IpxeScriptUrl) {
		return nil, false
	}
	return o.IpxeScriptUrl, true
}

// HasIpxeScriptUrl returns a boolean if a field has been set.
func (o *Device) HasIpxeScriptUrl() bool {
	if o != nil && !IsNil(o.IpxeScriptUrl) {
		return true
	}

	return false
}

// SetIpxeScriptUrl gets a reference to the given string and assigns it to the IpxeScriptUrl field.
func (o *Device) SetIpxeScriptUrl(v string) {
	o.IpxeScriptUrl = &v
}

// GetIqn returns the Iqn field value if set, zero value otherwise.
func (o *Device) GetIqn() string {
	if o == nil || IsNil(o.Iqn) {
		var ret string
		return ret
	}
	return *o.Iqn
}

// GetIqnOk returns a tuple with the Iqn field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetIqnOk() (*string, bool) {
	if o == nil || IsNil(o.Iqn) {
		return nil, false
	}
	return o.Iqn, true
}

// HasIqn returns a boolean if a field has been set.
func (o *Device) HasIqn() bool {
	if o != nil && !IsNil(o.Iqn) {
		return true
	}

	return false
}

// SetIqn gets a reference to the given string and assigns it to the Iqn field.
func (o *Device) SetIqn(v string) {
	o.Iqn = &v
}

// GetLocked returns the Locked field value if set, zero value otherwise.
func (o *Device) GetLocked() bool {
	if o == nil || IsNil(o.Locked) {
		var ret bool
		return ret
	}
	return *o.Locked
}

// GetLockedOk returns a tuple with the Locked field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetLockedOk() (*bool, bool) {
	if o == nil || IsNil(o.Locked) {
		return nil, false
	}
	return o.Locked, true
}

// HasLocked returns a boolean if a field has been set.
func (o *Device) HasLocked() bool {
	if o != nil && !IsNil(o.Locked) {
		return true
	}

	return false
}

// SetLocked gets a reference to the given bool and assigns it to the Locked field.
func (o *Device) SetLocked(v bool) {
	o.Locked = &v
}

// GetMetro returns the Metro field value if set, zero value otherwise.
func (o *Device) GetMetro() DeviceMetro {
	if o == nil || IsNil(o.Metro) {
		var ret DeviceMetro
		return ret
	}
	return *o.Metro
}

// GetMetroOk returns a tuple with the Metro field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetMetroOk() (*DeviceMetro, bool) {
	if o == nil || IsNil(o.Metro) {
		return nil, false
	}
	return o.Metro, true
}

// HasMetro returns a boolean if a field has been set.
func (o *Device) HasMetro() bool {
	if o != nil && !IsNil(o.Metro) {
		return true
	}

	return false
}

// SetMetro gets a reference to the given DeviceMetro and assigns it to the Metro field.
func (o *Device) SetMetro(v DeviceMetro) {
	o.Metro = &v
}

// GetNetworkFrozen returns the NetworkFrozen field value if set, zero value otherwise.
func (o *Device) GetNetworkFrozen() bool {
	if o == nil || IsNil(o.NetworkFrozen) {
		var ret bool
		return ret
	}
	return *o.NetworkFrozen
}

// GetNetworkFrozenOk returns a tuple with the NetworkFrozen field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetNetworkFrozenOk() (*bool, bool) {
	if o == nil || IsNil(o.NetworkFrozen) {
		return nil, false
	}
	return o.NetworkFrozen, true
}

// HasNetworkFrozen returns a boolean if a field has been set.
func (o *Device) HasNetworkFrozen() bool {
	if o != nil && !IsNil(o.NetworkFrozen) {
		return true
	}

	return false
}

// SetNetworkFrozen gets a reference to the given bool and assigns it to the NetworkFrozen field.
func (o *Device) SetNetworkFrozen(v bool) {
	o.NetworkFrozen = &v
}

// GetNetworkPorts returns the NetworkPorts field value if set, zero value otherwise.
func (o *Device) GetNetworkPorts() []Port {
	if o == nil || IsNil(o.NetworkPorts) {
		var ret []Port
		return ret
	}
	return o.NetworkPorts
}

// GetNetworkPortsOk returns a tuple with the NetworkPorts field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetNetworkPortsOk() ([]Port, bool) {
	if o == nil || IsNil(o.NetworkPorts) {
		return nil, false
	}
	return o.NetworkPorts, true
}

// HasNetworkPorts returns a boolean if a field has been set.
func (o *Device) HasNetworkPorts() bool {
	if o != nil && !IsNil(o.NetworkPorts) {
		return true
	}

	return false
}

// SetNetworkPorts gets a reference to the given []Port and assigns it to the NetworkPorts field.
func (o *Device) SetNetworkPorts(v []Port) {
	o.NetworkPorts = v
}

// GetOperatingSystem returns the OperatingSystem field value if set, zero value otherwise.
func (o *Device) GetOperatingSystem() OperatingSystem {
	if o == nil || IsNil(o.OperatingSystem) {
		var ret OperatingSystem
		return ret
	}
	return *o.OperatingSystem
}

// GetOperatingSystemOk returns a tuple with the OperatingSystem field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetOperatingSystemOk() (*OperatingSystem, bool) {
	if o == nil || IsNil(o.OperatingSystem) {
		return nil, false
	}
	return o.OperatingSystem, true
}

// HasOperatingSystem returns a boolean if a field has been set.
func (o *Device) HasOperatingSystem() bool {
	if o != nil && !IsNil(o.OperatingSystem) {
		return true
	}

	return false
}

// SetOperatingSystem gets a reference to the given OperatingSystem and assigns it to the OperatingSystem field.
func (o *Device) SetOperatingSystem(v OperatingSystem) {
	o.OperatingSystem = &v
}

// GetActions returns the Actions field value if set, zero value otherwise.
func (o *Device) GetActions() []DeviceActionsInner {
	if o == nil || IsNil(o.Actions) {
		var ret []DeviceActionsInner
		return ret
	}
	return o.Actions
}

// GetActionsOk returns a tuple with the Actions field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetActionsOk() ([]DeviceActionsInner, bool) {
	if o == nil || IsNil(o.Actions) {
		return nil, false
	}
	return o.Actions, true
}

// HasActions returns a boolean if a field has been set.
func (o *Device) HasActions() bool {
	if o != nil && !IsNil(o.Actions) {
		return true
	}

	return false
}

// SetActions gets a reference to the given []DeviceActionsInner and assigns it to the Actions field.
func (o *Device) SetActions(v []DeviceActionsInner) {
	o.Actions = v
}

// GetPlan returns the Plan field value if set, zero value otherwise.
func (o *Device) GetPlan() Plan {
	if o == nil || IsNil(o.Plan) {
		var ret Plan
		return ret
	}
	return *o.Plan
}

// GetPlanOk returns a tuple with the Plan field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetPlanOk() (*Plan, bool) {
	if o == nil || IsNil(o.Plan) {
		return nil, false
	}
	return o.Plan, true
}

// HasPlan returns a boolean if a field has been set.
func (o *Device) HasPlan() bool {
	if o != nil && !IsNil(o.Plan) {
		return true
	}

	return false
}

// SetPlan gets a reference to the given Plan and assigns it to the Plan field.
func (o *Device) SetPlan(v Plan) {
	o.Plan = &v
}

// GetProject returns the Project field value if set, zero value otherwise.
func (o *Device) GetProject() Project {
	if o == nil || IsNil(o.Project) {
		var ret Project
		return ret
	}
	return *o.Project
}

// GetProjectOk returns a tuple with the Project field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetProjectOk() (*Project, bool) {
	if o == nil || IsNil(o.Project) {
		return nil, false
	}
	return o.Project, true
}

// HasProject returns a boolean if a field has been set.
func (o *Device) HasProject() bool {
	if o != nil && !IsNil(o.Project) {
		return true
	}

	return false
}

// SetProject gets a reference to the given Project and assigns it to the Project field.
func (o *Device) SetProject(v Project) {
	o.Project = &v
}

// GetProjectLite returns the ProjectLite field value if set, zero value otherwise.
func (o *Device) GetProjectLite() DeviceProjectLite {
	if o == nil || IsNil(o.ProjectLite) {
		var ret DeviceProjectLite
		return ret
	}
	return *o.ProjectLite
}

// GetProjectLiteOk returns a tuple with the ProjectLite field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetProjectLiteOk() (*DeviceProjectLite, bool) {
	if o == nil || IsNil(o.ProjectLite) {
		return nil, false
	}
	return o.ProjectLite, true
}

// HasProjectLite returns a boolean if a field has been set.
func (o *Device) HasProjectLite() bool {
	if o != nil && !IsNil(o.ProjectLite) {
		return true
	}

	return false
}

// SetProjectLite gets a reference to the given DeviceProjectLite and assigns it to the ProjectLite field.
func (o *Device) SetProjectLite(v DeviceProjectLite) {
	o.ProjectLite = &v
}

// GetProvisioningEvents returns the ProvisioningEvents field value if set, zero value otherwise.
func (o *Device) GetProvisioningEvents() []Event {
	if o == nil || IsNil(o.ProvisioningEvents) {
		var ret []Event
		return ret
	}
	return o.ProvisioningEvents
}

// GetProvisioningEventsOk returns a tuple with the ProvisioningEvents field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetProvisioningEventsOk() ([]Event, bool) {
	if o == nil || IsNil(o.ProvisioningEvents) {
		return nil, false
	}
	return o.ProvisioningEvents, true
}

// HasProvisioningEvents returns a boolean if a field has been set.
func (o *Device) HasProvisioningEvents() bool {
	if o != nil && !IsNil(o.ProvisioningEvents) {
		return true
	}

	return false
}

// SetProvisioningEvents gets a reference to the given []Event and assigns it to the ProvisioningEvents field.
func (o *Device) SetProvisioningEvents(v []Event) {
	o.ProvisioningEvents = v
}

// GetProvisioningPercentage returns the ProvisioningPercentage field value if set, zero value otherwise.
func (o *Device) GetProvisioningPercentage() float32 {
	if o == nil || IsNil(o.ProvisioningPercentage) {
		var ret float32
		return ret
	}
	return *o.ProvisioningPercentage
}

// GetProvisioningPercentageOk returns a tuple with the ProvisioningPercentage field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetProvisioningPercentageOk() (*float32, bool) {
	if o == nil || IsNil(o.ProvisioningPercentage) {
		return nil, false
	}
	return o.ProvisioningPercentage, true
}

// HasProvisioningPercentage returns a boolean if a field has been set.
func (o *Device) HasProvisioningPercentage() bool {
	if o != nil && !IsNil(o.ProvisioningPercentage) {
		return true
	}

	return false
}

// SetProvisioningPercentage gets a reference to the given float32 and assigns it to the ProvisioningPercentage field.
func (o *Device) SetProvisioningPercentage(v float32) {
	o.ProvisioningPercentage = &v
}

// GetRootPassword returns the RootPassword field value if set, zero value otherwise.
func (o *Device) GetRootPassword() string {
	if o == nil || IsNil(o.RootPassword) {
		var ret string
		return ret
	}
	return *o.RootPassword
}

// GetRootPasswordOk returns a tuple with the RootPassword field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetRootPasswordOk() (*string, bool) {
	if o == nil || IsNil(o.RootPassword) {
		return nil, false
	}
	return o.RootPassword, true
}

// HasRootPassword returns a boolean if a field has been set.
func (o *Device) HasRootPassword() bool {
	if o != nil && !IsNil(o.RootPassword) {
		return true
	}

	return false
}

// SetRootPassword gets a reference to the given string and assigns it to the RootPassword field.
func (o *Device) SetRootPassword(v string) {
	o.RootPassword = &v
}

// GetShortId returns the ShortId field value if set, zero value otherwise.
func (o *Device) GetShortId() string {
	if o == nil || IsNil(o.ShortId) {
		var ret string
		return ret
	}
	return *o.ShortId
}

// GetShortIdOk returns a tuple with the ShortId field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetShortIdOk() (*string, bool) {
	if o == nil || IsNil(o.ShortId) {
		return nil, false
	}
	return o.ShortId, true
}

// HasShortId returns a boolean if a field has been set.
func (o *Device) HasShortId() bool {
	if o != nil && !IsNil(o.ShortId) {
		return true
	}

	return false
}

// SetShortId gets a reference to the given string and assigns it to the ShortId field.
func (o *Device) SetShortId(v string) {
	o.ShortId = &v
}

// GetSpotInstance returns the SpotInstance field value if set, zero value otherwise.
func (o *Device) GetSpotInstance() bool {
	if o == nil || IsNil(o.SpotInstance) {
		var ret bool
		return ret
	}
	return *o.SpotInstance
}

// GetSpotInstanceOk returns a tuple with the SpotInstance field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetSpotInstanceOk() (*bool, bool) {
	if o == nil || IsNil(o.SpotInstance) {
		return nil, false
	}
	return o.SpotInstance, true
}

// HasSpotInstance returns a boolean if a field has been set.
func (o *Device) HasSpotInstance() bool {
	if o != nil && !IsNil(o.SpotInstance) {
		return true
	}

	return false
}

// SetSpotInstance gets a reference to the given bool and assigns it to the SpotInstance field.
func (o *Device) SetSpotInstance(v bool) {
	o.SpotInstance = &v
}

// GetSpotPriceMax returns the SpotPriceMax field value if set, zero value otherwise.
func (o *Device) GetSpotPriceMax() float32 {
	if o == nil || IsNil(o.SpotPriceMax) {
		var ret float32
		return ret
	}
	return *o.SpotPriceMax
}

// GetSpotPriceMaxOk returns a tuple with the SpotPriceMax field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetSpotPriceMaxOk() (*float32, bool) {
	if o == nil || IsNil(o.SpotPriceMax) {
		return nil, false
	}
	return o.SpotPriceMax, true
}

// HasSpotPriceMax returns a boolean if a field has been set.
func (o *Device) HasSpotPriceMax() bool {
	if o != nil && !IsNil(o.SpotPriceMax) {
		return true
	}

	return false
}

// SetSpotPriceMax gets a reference to the given float32 and assigns it to the SpotPriceMax field.
func (o *Device) SetSpotPriceMax(v float32) {
	o.SpotPriceMax = &v
}

// GetSshKeys returns the SshKeys field value if set, zero value otherwise.
func (o *Device) GetSshKeys() []Href {
	if o == nil || IsNil(o.SshKeys) {
		var ret []Href
		return ret
	}
	return o.SshKeys
}

// GetSshKeysOk returns a tuple with the SshKeys field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetSshKeysOk() ([]Href, bool) {
	if o == nil || IsNil(o.SshKeys) {
		return nil, false
	}
	return o.SshKeys, true
}

// HasSshKeys returns a boolean if a field has been set.
func (o *Device) HasSshKeys() bool {
	if o != nil && !IsNil(o.SshKeys) {
		return true
	}

	return false
}

// SetSshKeys gets a reference to the given []Href and assigns it to the SshKeys field.
func (o *Device) SetSshKeys(v []Href) {
	o.SshKeys = v
}

// GetState returns the State field value if set, zero value otherwise.
func (o *Device) GetState() DeviceState {
	if o == nil || IsNil(o.State) {
		var ret DeviceState
		return ret
	}
	return *o.State
}

// GetStateOk returns a tuple with the State field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetStateOk() (*DeviceState, bool) {
	if o == nil || IsNil(o.State) {
		return nil, false
	}
	return o.State, true
}

// HasState returns a boolean if a field has been set.
func (o *Device) HasState() bool {
	if o != nil && !IsNil(o.State) {
		return true
	}

	return false
}

// SetState gets a reference to the given DeviceState and assigns it to the State field.
func (o *Device) SetState(v DeviceState) {
	o.State = &v
}

// GetStorage returns the Storage field value if set, zero value otherwise.
func (o *Device) GetStorage() Storage {
	if o == nil || IsNil(o.Storage) {
		var ret Storage
		return ret
	}
	return *o.Storage
}

// GetStorageOk returns a tuple with the Storage field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetStorageOk() (*Storage, bool) {
	if o == nil || IsNil(o.Storage) {
		return nil, false
	}
	return o.Storage, true
}

// HasStorage returns a boolean if a field has been set.
func (o *Device) HasStorage() bool {
	if o != nil && !IsNil(o.Storage) {
		return true
	}

	return false
}

// SetStorage gets a reference to the given Storage and assigns it to the Storage field.
func (o *Device) SetStorage(v Storage) {
	o.Storage = &v
}

// GetSwitchUuid returns the SwitchUuid field value if set, zero value otherwise.
func (o *Device) GetSwitchUuid() string {
	if o == nil || IsNil(o.SwitchUuid) {
		var ret string
		return ret
	}
	return *o.SwitchUuid
}

// GetSwitchUuidOk returns a tuple with the SwitchUuid field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetSwitchUuidOk() (*string, bool) {
	if o == nil || IsNil(o.SwitchUuid) {
		return nil, false
	}
	return o.SwitchUuid, true
}

// HasSwitchUuid returns a boolean if a field has been set.
func (o *Device) HasSwitchUuid() bool {
	if o != nil && !IsNil(o.SwitchUuid) {
		return true
	}

	return false
}

// SetSwitchUuid gets a reference to the given string and assigns it to the SwitchUuid field.
func (o *Device) SetSwitchUuid(v string) {
	o.SwitchUuid = &v
}

// GetTags returns the Tags field value if set, zero value otherwise.
func (o *Device) GetTags() []string {
	if o == nil || IsNil(o.Tags) {
		var ret []string
		return ret
	}
	return o.Tags
}

// GetTagsOk returns a tuple with the Tags field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetTagsOk() ([]string, bool) {
	if o == nil || IsNil(o.Tags) {
		return nil, false
	}
	return o.Tags, true
}

// HasTags returns a boolean if a field has been set.
func (o *Device) HasTags() bool {
	if o != nil && !IsNil(o.Tags) {
		return true
	}

	return false
}

// SetTags gets a reference to the given []string and assigns it to the Tags field.
func (o *Device) SetTags(v []string) {
	o.Tags = v
}

// GetTerminationTime returns the TerminationTime field value if set, zero value otherwise.
func (o *Device) GetTerminationTime() time.Time {
	if o == nil || IsNil(o.TerminationTime) {
		var ret time.Time
		return ret
	}
	return *o.TerminationTime
}

// GetTerminationTimeOk returns a tuple with the TerminationTime field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetTerminationTimeOk() (*time.Time, bool) {
	if o == nil || IsNil(o.TerminationTime) {
		return nil, false
	}
	return o.TerminationTime, true
}

// HasTerminationTime returns a boolean if a field has been set.
func (o *Device) HasTerminationTime() bool {
	if o != nil && !IsNil(o.TerminationTime) {
		return true
	}

	return false
}

// SetTerminationTime gets a reference to the given time.Time and assigns it to the TerminationTime field.
func (o *Device) SetTerminationTime(v time.Time) {
	o.TerminationTime = &v
}

// GetUpdatedAt returns the UpdatedAt field value if set, zero value otherwise.
func (o *Device) GetUpdatedAt() time.Time {
	if o == nil || IsNil(o.UpdatedAt) {
		var ret time.Time
		return ret
	}
	return *o.UpdatedAt
}

// GetUpdatedAtOk returns a tuple with the UpdatedAt field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetUpdatedAtOk() (*time.Time, bool) {
	if o == nil || IsNil(o.UpdatedAt) {
		return nil, false
	}
	return o.UpdatedAt, true
}

// HasUpdatedAt returns a boolean if a field has been set.
func (o *Device) HasUpdatedAt() bool {
	if o != nil && !IsNil(o.UpdatedAt) {
		return true
	}

	return false
}

// SetUpdatedAt gets a reference to the given time.Time and assigns it to the UpdatedAt field.
func (o *Device) SetUpdatedAt(v time.Time) {
	o.UpdatedAt = &v
}

// GetUser returns the User field value if set, zero value otherwise.
func (o *Device) GetUser() string {
	if o == nil || IsNil(o.User) {
		var ret string
		return ret
	}
	return *o.User
}

// GetUserOk returns a tuple with the User field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetUserOk() (*string, bool) {
	if o == nil || IsNil(o.User) {
		return nil, false
	}
	return o.User, true
}

// HasUser returns a boolean if a field has been set.
func (o *Device) HasUser() bool {
	if o != nil && !IsNil(o.User) {
		return true
	}

	return false
}

// SetUser gets a reference to the given string and assigns it to the User field.
func (o *Device) SetUser(v string) {
	o.User = &v
}

// GetUserdata returns the Userdata field value if set, zero value otherwise.
func (o *Device) GetUserdata() string {
	if o == nil || IsNil(o.Userdata) {
		var ret string
		return ret
	}
	return *o.Userdata
}

// GetUserdataOk returns a tuple with the Userdata field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetUserdataOk() (*string, bool) {
	if o == nil || IsNil(o.Userdata) {
		return nil, false
	}
	return o.Userdata, true
}

// HasUserdata returns a boolean if a field has been set.
func (o *Device) HasUserdata() bool {
	if o != nil && !IsNil(o.Userdata) {
		return true
	}

	return false
}

// SetUserdata gets a reference to the given string and assigns it to the Userdata field.
func (o *Device) SetUserdata(v string) {
	o.Userdata = &v
}

// GetVolumes returns the Volumes field value if set, zero value otherwise.
func (o *Device) GetVolumes() []Href {
	if o == nil || IsNil(o.Volumes) {
		var ret []Href
		return ret
	}
	return o.Volumes
}

// GetVolumesOk returns a tuple with the Volumes field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetVolumesOk() ([]Href, bool) {
	if o == nil || IsNil(o.Volumes) {
		return nil, false
	}
	return o.Volumes, true
}

// HasVolumes returns a boolean if a field has been set.
func (o *Device) HasVolumes() bool {
	if o != nil && !IsNil(o.Volumes) {
		return true
	}

	return false
}

// SetVolumes gets a reference to the given []Href and assigns it to the Volumes field.
func (o *Device) SetVolumes(v []Href) {
	o.Volumes = v
}

// GetSos returns the Sos field value if set, zero value otherwise.
func (o *Device) GetSos() string {
	if o == nil || IsNil(o.Sos) {
		var ret string
		return ret
	}
	return *o.Sos
}

// GetSosOk returns a tuple with the Sos field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *Device) GetSosOk() (*string, bool) {
	if o == nil || IsNil(o.Sos) {
		return nil, false
	}
	return o.Sos, true
}

// HasSos returns a boolean if a field has been set.
func (o *Device) HasSos() bool {
	if o != nil && !IsNil(o.Sos) {
		return true
	}

	return false
}

// SetSos gets a reference to the given string and assigns it to the Sos field.
func (o *Device) SetSos(v string) {
	o.Sos = &v
}

func (o Device) MarshalJSON() ([]byte, error) {
	toSerialize, err := o.ToMap()
	if err != nil {
		return []byte{}, err
	}
	return json.Marshal(toSerialize)
}

func (o Device) ToMap() (map[string]interface{}, error) {
	toSerialize := map[string]interface{}{}
	if !IsNil(o.AlwaysPxe) {
		toSerialize["always_pxe"] = o.AlwaysPxe
	}
	if !IsNil(o.BillingCycle) {
		toSerialize["billing_cycle"] = o.BillingCycle
	}
	if !IsNil(o.BondingMode) {
		toSerialize["bonding_mode"] = o.BondingMode
	}
	if !IsNil(o.CreatedAt) {
		toSerialize["created_at"] = o.CreatedAt
	}
	if !IsNil(o.CreatedBy) {
		toSerialize["created_by"] = o.CreatedBy
	}
	if !IsNil(o.Customdata) {
		toSerialize["customdata"] = o.Customdata
	}
	if !IsNil(o.Description) {
		toSerialize["description"] = o.Description
	}
	if !IsNil(o.Facility) {
		toSerialize["facility"] = o.Facility
	}
	if !IsNil(o.FirmwareSetId) {
		toSerialize["firmware_set_id"] = o.FirmwareSetId
	}
	if !IsNil(o.HardwareReservation) {
		toSerialize["hardware_reservation"] = o.HardwareReservation
	}
	if !IsNil(o.Hostname) {
		toSerialize["hostname"] = o.Hostname
	}
	if !IsNil(o.Href) {
		toSerialize["href"] = o.Href
	}
	if !IsNil(o.Id) {
		toSerialize["id"] = o.Id
	}
	if !IsNil(o.ImageUrl) {
		toSerialize["image_url"] = o.ImageUrl
	}
	if !IsNil(o.IpAddresses) {
		toSerialize["ip_addresses"] = o.IpAddresses
	}
	if !IsNil(o.IpxeScriptUrl) {
		toSerialize["ipxe_script_url"] = o.IpxeScriptUrl
	}
	if !IsNil(o.Iqn) {
		toSerialize["iqn"] = o.Iqn
	}
	if !IsNil(o.Locked) {
		toSerialize["locked"] = o.Locked
	}
	if !IsNil(o.Metro) {
		toSerialize["metro"] = o.Metro
	}
	if !IsNil(o.NetworkFrozen) {
		toSerialize["network_frozen"] = o.NetworkFrozen
	}
	if !IsNil(o.NetworkPorts) {
		toSerialize["network_ports"] = o.NetworkPorts
	}
	if !IsNil(o.OperatingSystem) {
		toSerialize["operating_system"] = o.OperatingSystem
	}
	if !IsNil(o.Actions) {
		toSerialize["actions"] = o.Actions
	}
	if !IsNil(o.Plan) {
		toSerialize["plan"] = o.Plan
	}
	if !IsNil(o.Project) {
		toSerialize["project"] = o.Project
	}
	if !IsNil(o.ProjectLite) {
		toSerialize["project_lite"] = o.ProjectLite
	}
	if !IsNil(o.ProvisioningEvents) {
		toSerialize["provisioning_events"] = o.ProvisioningEvents
	}
	if !IsNil(o.ProvisioningPercentage) {
		toSerialize["provisioning_percentage"] = o.ProvisioningPercentage
	}
	if !IsNil(o.RootPassword) {
		toSerialize["root_password"] = o.RootPassword
	}
	if !IsNil(o.ShortId) {
		toSerialize["short_id"] = o.ShortId
	}
	if !IsNil(o.SpotInstance) {
		toSerialize["spot_instance"] = o.SpotInstance
	}
	if !IsNil(o.SpotPriceMax) {
		toSerialize["spot_price_max"] = o.SpotPriceMax
	}
	if !IsNil(o.SshKeys) {
		toSerialize["ssh_keys"] = o.SshKeys
	}
	if !IsNil(o.State) {
		toSerialize["state"] = o.State
	}
	if !IsNil(o.Storage) {
		toSerialize["storage"] = o.Storage
	}
	if !IsNil(o.SwitchUuid) {
		toSerialize["switch_uuid"] = o.SwitchUuid
	}
	if !IsNil(o.Tags) {
		toSerialize["tags"] = o.Tags
	}
	if !IsNil(o.TerminationTime) {
		toSerialize["termination_time"] = o.TerminationTime
	}
	if !IsNil(o.UpdatedAt) {
		toSerialize["updated_at"] = o.UpdatedAt
	}
	if !IsNil(o.User) {
		toSerialize["user"] = o.User
	}
	if !IsNil(o.Userdata) {
		toSerialize["userdata"] = o.Userdata
	}
	if !IsNil(o.Volumes) {
		toSerialize["volumes"] = o.Volumes
	}
	if !IsNil(o.Sos) {
		toSerialize["sos"] = o.Sos
	}

	for key, value := range o.AdditionalProperties {
		toSerialize[key] = value
	}

	return toSerialize, nil
}

func (o *Device) UnmarshalJSON(data []byte) (err error) {
	varDevice := _Device{}

	err = json.Unmarshal(data, &varDevice)

	if err != nil {
		return err
	}

	*o = Device(varDevice)

	additionalProperties := make(map[string]interface{})

	if err = json.Unmarshal(data, &additionalProperties); err == nil {
		delete(additionalProperties, "always_pxe")
		delete(additionalProperties, "billing_cycle")
		delete(additionalProperties, "bonding_mode")
		delete(additionalProperties, "created_at")
		delete(additionalProperties, "created_by")
		delete(additionalProperties, "customdata")
		delete(additionalProperties, "description")
		delete(additionalProperties, "facility")
		delete(additionalProperties, "firmware_set_id")
		delete(additionalProperties, "hardware_reservation")
		delete(additionalProperties, "hostname")
		delete(additionalProperties, "href")
		delete(additionalProperties, "id")
		delete(additionalProperties, "image_url")
		delete(additionalProperties, "ip_addresses")
		delete(additionalProperties, "ipxe_script_url")
		delete(additionalProperties, "iqn")
		delete(additionalProperties, "locked")
		delete(additionalProperties, "metro")
		delete(additionalProperties, "network_frozen")
		delete(additionalProperties, "network_ports")
		delete(additionalProperties, "operating_system")
		delete(additionalProperties, "actions")
		delete(additionalProperties, "plan")
		delete(additionalProperties, "project")
		delete(additionalProperties, "project_lite")
		delete(additionalProperties, "provisioning_events")
		delete(additionalProperties, "provisioning_percentage")
		delete(additionalProperties, "root_password")
		delete(additionalProperties, "short_id")
		delete(additionalProperties, "spot_instance")
		delete(additionalProperties, "spot_price_max")
		delete(additionalProperties, "ssh_keys")
		delete(additionalProperties, "state")
		delete(additionalProperties, "storage")
		delete(additionalProperties, "switch_uuid")
		delete(additionalProperties, "tags")
		delete(additionalProperties, "termination_time")
		delete(additionalProperties, "updated_at")
		delete(additionalProperties, "user")
		delete(additionalProperties, "userdata")
		delete(additionalProperties, "volumes")
		delete(additionalProperties, "sos")
		o.AdditionalProperties = additionalProperties
	}

	return err
}

type NullableDevice struct {
	value *Device
	isSet bool
}

func (v NullableDevice) Get() *Device {
	return v.value
}

func (v *NullableDevice) Set(val *Device) {
	v.value = val
	v.isSet = true
}

func (v NullableDevice) IsSet() bool {
	return v.isSet
}

func (v *NullableDevice) Unset() {
	v.value = nil
	v.isSet = false
}

func NewNullableDevice(val *Device) *NullableDevice {
	return &NullableDevice{value: val, isSet: true}
}

func (v NullableDevice) MarshalJSON() ([]byte, error) {
	return json.Marshal(v.value)
}

func (v *NullableDevice) UnmarshalJSON(src []byte) error {
	v.isSet = true
	return json.Unmarshal(src, &v.value)
}
