/*
Metal API

# Introduction Equinix Metal provides a RESTful HTTP API which can be reached at <https://api.equinix.com/metal/v1>. This document describes the API and how to use it.  The API allows you to programmatically interact with all of your Equinix Metal resources, including devices, networks, addresses, organizations, projects, and your user account. Every feature of the Equinix Metal web interface is accessible through the API.  The API docs are generated from the Equinix Metal OpenAPI specification and are officially hosted at <https://metal.equinix.com/developers/api>.  # Common Parameters  The Equinix Metal API uses a few methods to minimize network traffic and improve throughput. These parameters are not used in all API calls, but are used often enough to warrant their own section. Look for these parameters in the documentation for the API calls that support them.  ## Pagination  Pagination is used to limit the number of results returned in a single request. The API will return a maximum of 100 results per page. To retrieve additional results, you can use the `page` and `per_page` query parameters.  The `page` parameter is used to specify the page number. The first page is `1`. The `per_page` parameter is used to specify the number of results per page. The maximum number of results differs by resource type.  ## Sorting  Where offered, the API allows you to sort results by a specific field. To sort results use the `sort_by` query parameter with the root level field name as the value. The `sort_direction` parameter is used to specify the sort direction, either either `asc` (ascending) or `desc` (descending).  ## Filtering  Filtering is used to limit the results returned in a single request. The API supports filtering by certain fields in the response. To filter results, you can use the field as a query parameter.  For example, to filter the IP list to only return public IPv4 addresses, you can filter by the `type` field, as in the following request:  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/projects/id/ips?type=public_ipv4 ```  Only IP addresses with the `type` field set to `public_ipv4` will be returned.  ## Searching  Searching is used to find matching resources using multiple field comparissons. The API supports searching in resources that define this behavior. Currently the search parameter is only available on devices, ssh_keys, api_keys and memberships endpoints.  To search resources you can use the `search` query parameter.  ## Include and Exclude  For resources that contain references to other resources, sucha as a Device that refers to the Project it resides in, the Equinix Metal API will returns `href` values (API links) to the associated resource.  ```json {   ...   \"project\": {     \"href\": \"/metal/v1/projects/f3f131c8-f302-49ef-8c44-9405022dc6dd\"   } } ```  If you're going need the project details, you can avoid a second API request.  Specify the contained `href` resources and collections that you'd like to have included in the response using the `include` query parameter.  For example:  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=projects ```  The `include` parameter is generally accepted in `GET`, `POST`, `PUT`, and `PATCH` requests where `href` resources are presented.  To have multiple resources include, use a comma-separated list (e.g. `?include=emails,projects,memberships`).  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=emails,projects,memberships ```  You may also include nested associations up to three levels deep using dot notation (`?include=memberships.projects`):  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=memberships.projects ```  To exclude resources, and optimize response delivery, use the `exclude` query parameter. The `exclude` parameter is generally accepted in `GET`, `POST`, `PUT`, and `PATCH` requests for fields with nested object responses. When excluded, these fields will be replaced with an object that contains only an `href` field.

API version: 1.0.0
Contact: support@equinixmetal.com
*/

// Code generated by OpenAPI Generator (https://openapi-generator.tech); DO NOT EDIT.

package metalv1

import (
	"encoding/json"
)

// checks if the SpotMarketPricesPerMetroReport type satisfies the MappedNullable interface at compile time
var _ MappedNullable = &SpotMarketPricesPerMetroReport{}

// SpotMarketPricesPerMetroReport struct for SpotMarketPricesPerMetroReport
type SpotMarketPricesPerMetroReport struct {
	Am                   *SpotPricesPerFacility `json:"am,omitempty"`
	Ch                   *SpotPricesPerFacility `json:"ch,omitempty"`
	Da                   *SpotPricesPerFacility `json:"da,omitempty"`
	La                   *SpotPricesPerFacility `json:"la,omitempty"`
	Ny                   *SpotPricesPerFacility `json:"ny,omitempty"`
	Sg                   *SpotPricesPerFacility `json:"sg,omitempty"`
	Sv                   *SpotPricesPerFacility `json:"sv,omitempty"`
	AdditionalProperties map[string]interface{}
}

type _SpotMarketPricesPerMetroReport SpotMarketPricesPerMetroReport

// NewSpotMarketPricesPerMetroReport instantiates a new SpotMarketPricesPerMetroReport object
// This constructor will assign default values to properties that have it defined,
// and makes sure properties required by API are set, but the set of arguments
// will change when the set of required properties is changed
func NewSpotMarketPricesPerMetroReport() *SpotMarketPricesPerMetroReport {
	this := SpotMarketPricesPerMetroReport{}
	return &this
}

// NewSpotMarketPricesPerMetroReportWithDefaults instantiates a new SpotMarketPricesPerMetroReport object
// This constructor will only assign default values to properties that have it defined,
// but it doesn't guarantee that properties required by API are set
func NewSpotMarketPricesPerMetroReportWithDefaults() *SpotMarketPricesPerMetroReport {
	this := SpotMarketPricesPerMetroReport{}
	return &this
}

// GetAm returns the Am field value if set, zero value otherwise.
func (o *SpotMarketPricesPerMetroReport) GetAm() SpotPricesPerFacility {
	if o == nil || IsNil(o.Am) {
		var ret SpotPricesPerFacility
		return ret
	}
	return *o.Am
}

// GetAmOk returns a tuple with the Am field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *SpotMarketPricesPerMetroReport) GetAmOk() (*SpotPricesPerFacility, bool) {
	if o == nil || IsNil(o.Am) {
		return nil, false
	}
	return o.Am, true
}

// HasAm returns a boolean if a field has been set.
func (o *SpotMarketPricesPerMetroReport) HasAm() bool {
	if o != nil && !IsNil(o.Am) {
		return true
	}

	return false
}

// SetAm gets a reference to the given SpotPricesPerFacility and assigns it to the Am field.
func (o *SpotMarketPricesPerMetroReport) SetAm(v SpotPricesPerFacility) {
	o.Am = &v
}

// GetCh returns the Ch field value if set, zero value otherwise.
func (o *SpotMarketPricesPerMetroReport) GetCh() SpotPricesPerFacility {
	if o == nil || IsNil(o.Ch) {
		var ret SpotPricesPerFacility
		return ret
	}
	return *o.Ch
}

// GetChOk returns a tuple with the Ch field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *SpotMarketPricesPerMetroReport) GetChOk() (*SpotPricesPerFacility, bool) {
	if o == nil || IsNil(o.Ch) {
		return nil, false
	}
	return o.Ch, true
}

// HasCh returns a boolean if a field has been set.
func (o *SpotMarketPricesPerMetroReport) HasCh() bool {
	if o != nil && !IsNil(o.Ch) {
		return true
	}

	return false
}

// SetCh gets a reference to the given SpotPricesPerFacility and assigns it to the Ch field.
func (o *SpotMarketPricesPerMetroReport) SetCh(v SpotPricesPerFacility) {
	o.Ch = &v
}

// GetDa returns the Da field value if set, zero value otherwise.
func (o *SpotMarketPricesPerMetroReport) GetDa() SpotPricesPerFacility {
	if o == nil || IsNil(o.Da) {
		var ret SpotPricesPerFacility
		return ret
	}
	return *o.Da
}

// GetDaOk returns a tuple with the Da field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *SpotMarketPricesPerMetroReport) GetDaOk() (*SpotPricesPerFacility, bool) {
	if o == nil || IsNil(o.Da) {
		return nil, false
	}
	return o.Da, true
}

// HasDa returns a boolean if a field has been set.
func (o *SpotMarketPricesPerMetroReport) HasDa() bool {
	if o != nil && !IsNil(o.Da) {
		return true
	}

	return false
}

// SetDa gets a reference to the given SpotPricesPerFacility and assigns it to the Da field.
func (o *SpotMarketPricesPerMetroReport) SetDa(v SpotPricesPerFacility) {
	o.Da = &v
}

// GetLa returns the La field value if set, zero value otherwise.
func (o *SpotMarketPricesPerMetroReport) GetLa() SpotPricesPerFacility {
	if o == nil || IsNil(o.La) {
		var ret SpotPricesPerFacility
		return ret
	}
	return *o.La
}

// GetLaOk returns a tuple with the La field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *SpotMarketPricesPerMetroReport) GetLaOk() (*SpotPricesPerFacility, bool) {
	if o == nil || IsNil(o.La) {
		return nil, false
	}
	return o.La, true
}

// HasLa returns a boolean if a field has been set.
func (o *SpotMarketPricesPerMetroReport) HasLa() bool {
	if o != nil && !IsNil(o.La) {
		return true
	}

	return false
}

// SetLa gets a reference to the given SpotPricesPerFacility and assigns it to the La field.
func (o *SpotMarketPricesPerMetroReport) SetLa(v SpotPricesPerFacility) {
	o.La = &v
}

// GetNy returns the Ny field value if set, zero value otherwise.
func (o *SpotMarketPricesPerMetroReport) GetNy() SpotPricesPerFacility {
	if o == nil || IsNil(o.Ny) {
		var ret SpotPricesPerFacility
		return ret
	}
	return *o.Ny
}

// GetNyOk returns a tuple with the Ny field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *SpotMarketPricesPerMetroReport) GetNyOk() (*SpotPricesPerFacility, bool) {
	if o == nil || IsNil(o.Ny) {
		return nil, false
	}
	return o.Ny, true
}

// HasNy returns a boolean if a field has been set.
func (o *SpotMarketPricesPerMetroReport) HasNy() bool {
	if o != nil && !IsNil(o.Ny) {
		return true
	}

	return false
}

// SetNy gets a reference to the given SpotPricesPerFacility and assigns it to the Ny field.
func (o *SpotMarketPricesPerMetroReport) SetNy(v SpotPricesPerFacility) {
	o.Ny = &v
}

// GetSg returns the Sg field value if set, zero value otherwise.
func (o *SpotMarketPricesPerMetroReport) GetSg() SpotPricesPerFacility {
	if o == nil || IsNil(o.Sg) {
		var ret SpotPricesPerFacility
		return ret
	}
	return *o.Sg
}

// GetSgOk returns a tuple with the Sg field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *SpotMarketPricesPerMetroReport) GetSgOk() (*SpotPricesPerFacility, bool) {
	if o == nil || IsNil(o.Sg) {
		return nil, false
	}
	return o.Sg, true
}

// HasSg returns a boolean if a field has been set.
func (o *SpotMarketPricesPerMetroReport) HasSg() bool {
	if o != nil && !IsNil(o.Sg) {
		return true
	}

	return false
}

// SetSg gets a reference to the given SpotPricesPerFacility and assigns it to the Sg field.
func (o *SpotMarketPricesPerMetroReport) SetSg(v SpotPricesPerFacility) {
	o.Sg = &v
}

// GetSv returns the Sv field value if set, zero value otherwise.
func (o *SpotMarketPricesPerMetroReport) GetSv() SpotPricesPerFacility {
	if o == nil || IsNil(o.Sv) {
		var ret SpotPricesPerFacility
		return ret
	}
	return *o.Sv
}

// GetSvOk returns a tuple with the Sv field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *SpotMarketPricesPerMetroReport) GetSvOk() (*SpotPricesPerFacility, bool) {
	if o == nil || IsNil(o.Sv) {
		return nil, false
	}
	return o.Sv, true
}

// HasSv returns a boolean if a field has been set.
func (o *SpotMarketPricesPerMetroReport) HasSv() bool {
	if o != nil && !IsNil(o.Sv) {
		return true
	}

	return false
}

// SetSv gets a reference to the given SpotPricesPerFacility and assigns it to the Sv field.
func (o *SpotMarketPricesPerMetroReport) SetSv(v SpotPricesPerFacility) {
	o.Sv = &v
}

func (o SpotMarketPricesPerMetroReport) MarshalJSON() ([]byte, error) {
	toSerialize, err := o.ToMap()
	if err != nil {
		return []byte{}, err
	}
	return json.Marshal(toSerialize)
}

func (o SpotMarketPricesPerMetroReport) ToMap() (map[string]interface{}, error) {
	toSerialize := map[string]interface{}{}
	if !IsNil(o.Am) {
		toSerialize["am"] = o.Am
	}
	if !IsNil(o.Ch) {
		toSerialize["ch"] = o.Ch
	}
	if !IsNil(o.Da) {
		toSerialize["da"] = o.Da
	}
	if !IsNil(o.La) {
		toSerialize["la"] = o.La
	}
	if !IsNil(o.Ny) {
		toSerialize["ny"] = o.Ny
	}
	if !IsNil(o.Sg) {
		toSerialize["sg"] = o.Sg
	}
	if !IsNil(o.Sv) {
		toSerialize["sv"] = o.Sv
	}

	for key, value := range o.AdditionalProperties {
		toSerialize[key] = value
	}

	return toSerialize, nil
}

func (o *SpotMarketPricesPerMetroReport) UnmarshalJSON(data []byte) (err error) {
	varSpotMarketPricesPerMetroReport := _SpotMarketPricesPerMetroReport{}

	err = json.Unmarshal(data, &varSpotMarketPricesPerMetroReport)

	if err != nil {
		return err
	}

	*o = SpotMarketPricesPerMetroReport(varSpotMarketPricesPerMetroReport)

	additionalProperties := make(map[string]interface{})

	if err = json.Unmarshal(data, &additionalProperties); err == nil {
		delete(additionalProperties, "am")
		delete(additionalProperties, "ch")
		delete(additionalProperties, "da")
		delete(additionalProperties, "la")
		delete(additionalProperties, "ny")
		delete(additionalProperties, "sg")
		delete(additionalProperties, "sv")
		o.AdditionalProperties = additionalProperties
	}

	return err
}

type NullableSpotMarketPricesPerMetroReport struct {
	value *SpotMarketPricesPerMetroReport
	isSet bool
}

func (v NullableSpotMarketPricesPerMetroReport) Get() *SpotMarketPricesPerMetroReport {
	return v.value
}

func (v *NullableSpotMarketPricesPerMetroReport) Set(val *SpotMarketPricesPerMetroReport) {
	v.value = val
	v.isSet = true
}

func (v NullableSpotMarketPricesPerMetroReport) IsSet() bool {
	return v.isSet
}

func (v *NullableSpotMarketPricesPerMetroReport) Unset() {
	v.value = nil
	v.isSet = false
}

func NewNullableSpotMarketPricesPerMetroReport(val *SpotMarketPricesPerMetroReport) *NullableSpotMarketPricesPerMetroReport {
	return &NullableSpotMarketPricesPerMetroReport{value: val, isSet: true}
}

func (v NullableSpotMarketPricesPerMetroReport) MarshalJSON() ([]byte, error) {
	return json.Marshal(v.value)
}

func (v *NullableSpotMarketPricesPerMetroReport) UnmarshalJSON(src []byte) error {
	v.isSet = true
	return json.Unmarshal(src, &v.value)
}
