/*
Metal API

# Introduction Equinix Metal provides a RESTful HTTP API which can be reached at <https://api.equinix.com/metal/v1>. This document describes the API and how to use it.  The API allows you to programmatically interact with all of your Equinix Metal resources, including devices, networks, addresses, organizations, projects, and your user account. Every feature of the Equinix Metal web interface is accessible through the API.  The API docs are generated from the Equinix Metal OpenAPI specification and are officially hosted at <https://metal.equinix.com/developers/api>.  # Common Parameters  The Equinix Metal API uses a few methods to minimize network traffic and improve throughput. These parameters are not used in all API calls, but are used often enough to warrant their own section. Look for these parameters in the documentation for the API calls that support them.  ## Pagination  Pagination is used to limit the number of results returned in a single request. The API will return a maximum of 100 results per page. To retrieve additional results, you can use the `page` and `per_page` query parameters.  The `page` parameter is used to specify the page number. The first page is `1`. The `per_page` parameter is used to specify the number of results per page. The maximum number of results differs by resource type.  ## Sorting  Where offered, the API allows you to sort results by a specific field. To sort results use the `sort_by` query parameter with the root level field name as the value. The `sort_direction` parameter is used to specify the sort direction, either either `asc` (ascending) or `desc` (descending).  ## Filtering  Filtering is used to limit the results returned in a single request. The API supports filtering by certain fields in the response. To filter results, you can use the field as a query parameter.  For example, to filter the IP list to only return public IPv4 addresses, you can filter by the `type` field, as in the following request:  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/projects/id/ips?type=public_ipv4 ```  Only IP addresses with the `type` field set to `public_ipv4` will be returned.  ## Searching  Searching is used to find matching resources using multiple field comparissons. The API supports searching in resources that define this behavior. Currently the search parameter is only available on devices, ssh_keys, api_keys and memberships endpoints.  To search resources you can use the `search` query parameter.  ## Include and Exclude  For resources that contain references to other resources, sucha as a Device that refers to the Project it resides in, the Equinix Metal API will returns `href` values (API links) to the associated resource.  ```json {   ...   \"project\": {     \"href\": \"/metal/v1/projects/f3f131c8-f302-49ef-8c44-9405022dc6dd\"   } } ```  If you're going need the project details, you can avoid a second API request.  Specify the contained `href` resources and collections that you'd like to have included in the response using the `include` query parameter.  For example:  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=projects ```  The `include` parameter is generally accepted in `GET`, `POST`, `PUT`, and `PATCH` requests where `href` resources are presented.  To have multiple resources include, use a comma-separated list (e.g. `?include=emails,projects,memberships`).  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=emails,projects,memberships ```  You may also include nested associations up to three levels deep using dot notation (`?include=memberships.projects`):  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=memberships.projects ```  To exclude resources, and optimize response delivery, use the `exclude` query parameter. The `exclude` parameter is generally accepted in `GET`, `POST`, `PUT`, and `PATCH` requests for fields with nested object responses. When excluded, these fields will be replaced with an object that contains only an `href` field.

API version: 1.0.0
Contact: support@equinixmetal.com
*/

// Code generated by OpenAPI Generator (https://openapi-generator.tech); DO NOT EDIT.

package metalv1

import (
	"encoding/json"
	"fmt"
)

// checks if the VrfMetalGatewayCreateInput type satisfies the MappedNullable interface at compile time
var _ MappedNullable = &VrfMetalGatewayCreateInput{}

// VrfMetalGatewayCreateInput struct for VrfMetalGatewayCreateInput
type VrfMetalGatewayCreateInput struct {
	// The UUID an a VRF IP Reservation that belongs to the same project as the one in which the Metal Gateway is to be created. Additionally, the VRF IP Reservation and the Virtual Network must reside in the same Metro.
	IpReservationId string `json:"ip_reservation_id"`
	// The UUID of a Metro Virtual Network that belongs to the same project as the one in which the Metal Gateway is to be created. Additionally, the Virtual Network and the VRF IP Reservation must reside in the same metro. In the case of the IP reservation being an IPv6 based VRF IP Reservation, the Virtual Network must not already have an associated IPv6 based VRF IP Reservation. There can be exactly one IPv6 based VRF IP Reservation associated to a Virtual Network.
	VirtualNetworkId     string `json:"virtual_network_id"`
	AdditionalProperties map[string]interface{}
}

type _VrfMetalGatewayCreateInput VrfMetalGatewayCreateInput

// NewVrfMetalGatewayCreateInput instantiates a new VrfMetalGatewayCreateInput object
// This constructor will assign default values to properties that have it defined,
// and makes sure properties required by API are set, but the set of arguments
// will change when the set of required properties is changed
func NewVrfMetalGatewayCreateInput(ipReservationId string, virtualNetworkId string) *VrfMetalGatewayCreateInput {
	this := VrfMetalGatewayCreateInput{}
	this.IpReservationId = ipReservationId
	this.VirtualNetworkId = virtualNetworkId
	return &this
}

// NewVrfMetalGatewayCreateInputWithDefaults instantiates a new VrfMetalGatewayCreateInput object
// This constructor will only assign default values to properties that have it defined,
// but it doesn't guarantee that properties required by API are set
func NewVrfMetalGatewayCreateInputWithDefaults() *VrfMetalGatewayCreateInput {
	this := VrfMetalGatewayCreateInput{}
	return &this
}

// GetIpReservationId returns the IpReservationId field value
func (o *VrfMetalGatewayCreateInput) GetIpReservationId() string {
	if o == nil {
		var ret string
		return ret
	}

	return o.IpReservationId
}

// GetIpReservationIdOk returns a tuple with the IpReservationId field value
// and a boolean to check if the value has been set.
func (o *VrfMetalGatewayCreateInput) GetIpReservationIdOk() (*string, bool) {
	if o == nil {
		return nil, false
	}
	return &o.IpReservationId, true
}

// SetIpReservationId sets field value
func (o *VrfMetalGatewayCreateInput) SetIpReservationId(v string) {
	o.IpReservationId = v
}

// GetVirtualNetworkId returns the VirtualNetworkId field value
func (o *VrfMetalGatewayCreateInput) GetVirtualNetworkId() string {
	if o == nil {
		var ret string
		return ret
	}

	return o.VirtualNetworkId
}

// GetVirtualNetworkIdOk returns a tuple with the VirtualNetworkId field value
// and a boolean to check if the value has been set.
func (o *VrfMetalGatewayCreateInput) GetVirtualNetworkIdOk() (*string, bool) {
	if o == nil {
		return nil, false
	}
	return &o.VirtualNetworkId, true
}

// SetVirtualNetworkId sets field value
func (o *VrfMetalGatewayCreateInput) SetVirtualNetworkId(v string) {
	o.VirtualNetworkId = v
}

func (o VrfMetalGatewayCreateInput) MarshalJSON() ([]byte, error) {
	toSerialize, err := o.ToMap()
	if err != nil {
		return []byte{}, err
	}
	return json.Marshal(toSerialize)
}

func (o VrfMetalGatewayCreateInput) ToMap() (map[string]interface{}, error) {
	toSerialize := map[string]interface{}{}
	toSerialize["ip_reservation_id"] = o.IpReservationId
	toSerialize["virtual_network_id"] = o.VirtualNetworkId

	for key, value := range o.AdditionalProperties {
		toSerialize[key] = value
	}

	return toSerialize, nil
}

func (o *VrfMetalGatewayCreateInput) UnmarshalJSON(data []byte) (err error) {
	// This validates that all required properties are included in the JSON object
	// by unmarshalling the object into a generic map with string keys and checking
	// that every required field exists as a key in the generic map.
	requiredProperties := []string{
		"ip_reservation_id",
		"virtual_network_id",
	}

	allProperties := make(map[string]interface{})

	err = json.Unmarshal(data, &allProperties)

	if err != nil {
		return err
	}

	for _, requiredProperty := range requiredProperties {
		if _, exists := allProperties[requiredProperty]; !exists {
			return fmt.Errorf("no value given for required property %v", requiredProperty)
		}
	}

	varVrfMetalGatewayCreateInput := _VrfMetalGatewayCreateInput{}

	err = json.Unmarshal(data, &varVrfMetalGatewayCreateInput)

	if err != nil {
		return err
	}

	*o = VrfMetalGatewayCreateInput(varVrfMetalGatewayCreateInput)

	additionalProperties := make(map[string]interface{})

	if err = json.Unmarshal(data, &additionalProperties); err == nil {
		delete(additionalProperties, "ip_reservation_id")
		delete(additionalProperties, "virtual_network_id")
		o.AdditionalProperties = additionalProperties
	}

	return err
}

type NullableVrfMetalGatewayCreateInput struct {
	value *VrfMetalGatewayCreateInput
	isSet bool
}

func (v NullableVrfMetalGatewayCreateInput) Get() *VrfMetalGatewayCreateInput {
	return v.value
}

func (v *NullableVrfMetalGatewayCreateInput) Set(val *VrfMetalGatewayCreateInput) {
	v.value = val
	v.isSet = true
}

func (v NullableVrfMetalGatewayCreateInput) IsSet() bool {
	return v.isSet
}

func (v *NullableVrfMetalGatewayCreateInput) Unset() {
	v.value = nil
	v.isSet = false
}

func NewNullableVrfMetalGatewayCreateInput(val *VrfMetalGatewayCreateInput) *NullableVrfMetalGatewayCreateInput {
	return &NullableVrfMetalGatewayCreateInput{value: val, isSet: true}
}

func (v NullableVrfMetalGatewayCreateInput) MarshalJSON() ([]byte, error) {
	return json.Marshal(v.value)
}

func (v *NullableVrfMetalGatewayCreateInput) UnmarshalJSON(src []byte) error {
	v.isSet = true
	return json.Unmarshal(src, &v.value)
}
